package io.keikai.app;

import org.zkoss.lang.Library;
import org.zkoss.zk.ui.WebApps;

import java.util.Optional;

/**
 * List all library properties supported by this application.
 */
public class Configuration {
    /**
     * If true, then all UI about uploading or importing a file are disabled.
     */
    private static final String BOOKREPOSTORY_READONLY = "kkapp.bookrepostory.readonly";
    /**
     * If true, the application shows a confirmation dialog when a user close a tab if there is an unsaved change. Otherwise, it doesn't show any message.
     */
    private static final String WARNING_UNSAVED = "kkapp.warning.save";
    /**
     * If true, then a user doesn't edit a file with other users under collaboration mode. Each user opens a file in his own copy.
     */
    private static final String COLLABORATION_DISABLED = "kkapp.collaboration.disabled";
    /**
     * If true, then a string about "Keikai Spreadsheet [version edition]" doesn't show in the menu bar.
     */
    private static final String MENU_ZSSMARK_HIDDEN = "kkapp.menu.zssmark.hidden";
    /**
     * For customization. Specify a URL for a menu item "Help > About"
     */
    private static final String MENU_ABOUT_URL = "kkapp.menu.about.url";
    /**
     * If true, "Share" button in the menu bar is hidden.
     */
    private static final String MENU_SHAREBOOK_HIDDEN = "kkapp.menu.sharebook.hidden";
    /**
     * If true, "change username" icon in the menu bar is hidden.
     */
    private static final String MENU_USERNAME_HIDDEN = "kkapp.menu.username.hidden";
    /**
     * The path that this application looks up for files
     */
    private static final String REPOSITORY_ROOT = "kkapp.repository.root";

    public static final String DEFAULT_REPOSITORY_ROOT = "/WEB-INF/books/";

    /**
     * When an error occurs in an event listener, the error message showed to a user.
     */
    public static final String KKAPP_ERROR_HANDLING_MESSAGE = "kkapp.errorHandling.message";

    @Deprecated
    private static final String ZSSAPP_WARNING_SAVE = "zssapp.warning.save";
    @Deprecated
    private static final String ZSSAPP_COLLABORATION_DISABLED = "zssapp.collaboration.disabled";
    @Deprecated
    private static final String ZSSAPP_BOOKREPOSTORY_READONLY = "zssapp.bookrepostory.readonly";
    @Deprecated
    private static final String ZSSAPP_MENU_USERNAME_HIDDEN = "zssapp.menu.username.hidden";
    @Deprecated
    private static final String ZSSAPP_MENU_ABOUT_URL = "zssapp.menu.about.url";
    @Deprecated
    private static final String ZSSAPP_MENU_ZSSMARK_HIDDEN = "zssapp.menu.zssmark.hidden";
    @Deprecated
    private static final String ZSSAPP_MENU_SHAREBOOK_HIDDEN = "zssapp.menu.sharebook.hidden";
    @Deprecated
    private static final String ZSSAPP_REPOSITORY_ROOT = "zssapp.repository.root";

    public static boolean isRepositoryReadonly() {
        return Boolean.valueOf((Optional.ofNullable(Library.getProperty(BOOKREPOSTORY_READONLY))
                .orElse(Library.getProperty(ZSSAPP_BOOKREPOSTORY_READONLY, "false"))));
    }

    public static Boolean isShowUnsavedAlert() {
        return Boolean.valueOf(Optional.ofNullable(Library.getProperty(WARNING_UNSAVED))
                .orElse(Library.getProperty(ZSSAPP_WARNING_SAVE, "true")));
    }

    public static Boolean isCollaborationDisabled() {
        return Boolean.valueOf(Optional.ofNullable(Library.getProperty(COLLABORATION_DISABLED))
                .orElse(Library.getProperty(ZSSAPP_COLLABORATION_DISABLED)));
    }

    public static String getRepositoryRoot() {
        return Optional.ofNullable(Library.getProperty(REPOSITORY_ROOT))
                .orElse(Library.getProperty(ZSSAPP_REPOSITORY_ROOT, WebApps.getCurrent().getRealPath(DEFAULT_REPOSITORY_ROOT)));
    }

    public static Boolean isUsernameHidden() {
        return Boolean.valueOf(Optional.ofNullable(Library.getProperty(MENU_USERNAME_HIDDEN))
                .orElse(Library.getProperty(ZSSAPP_MENU_USERNAME_HIDDEN)));
    }

    public static Boolean isShareBookHidden() {
        return Boolean.valueOf(Optional.ofNullable(Library.getProperty(MENU_SHAREBOOK_HIDDEN))
                .orElse(Library.getProperty(ZSSAPP_MENU_SHAREBOOK_HIDDEN)));
    }

    public static String getAboutUrl() {
        return Optional.ofNullable(Library.getProperty(MENU_ABOUT_URL))
                .orElse(Library.getProperty(ZSSAPP_MENU_ABOUT_URL, "https://keikai.io"));
    }

    public static Boolean isSpreadsheetMarkHidden() {
        return Boolean.valueOf(Optional.ofNullable(Library.getProperty(MENU_ZSSMARK_HIDDEN))
                .orElse(Library.getProperty(ZSSAPP_MENU_ZSSMARK_HIDDEN)));
    }

    public static String getErrorHandlingMessage(){
        return Library.getProperty(KKAPP_ERROR_HANDLING_MESSAGE);
    }
}
