/* 
	Purpose:
		
	Description:
		
	History:
		2013/7/10, Created by dennis

Copyright (C) 2013 Potix Corporation. All Rights Reserved.

*/
package io.keikai.app.ui.dlg;

import io.keikai.api.Importers;
import io.keikai.api.model.Book;
import io.keikai.app.*;
import io.keikai.app.impl.*;
import io.keikai.app.repository.BookRepositoryFactory;
import io.keikai.app.repository.impl.BookUtil;
import io.keikai.app.ui.UiUtil;
import org.slf4j.*;
import org.zkoss.util.media.Media;
import org.zkoss.zk.ui.Executions;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zk.ui.event.*;
import org.zkoss.zk.ui.select.annotation.*;
import org.zkoss.zul.*;

import java.io.*;
import java.text.*;
import java.util.*;

/**
 * 
 * @author dennis
 *
 */
public class OpenManageBookCtrl extends DlgCtrlBase{
	private static final long serialVersionUID = 1L;
	
	public final static String ARG_BOOKINFO = "bookinfo";
	public final static String ARG_BOOK = "book";
	
	private final static String URI = "~./zssapp/dlg/openManageBook.zul";
	
	public static final String ON_OPEN = "onOpen";
	
	private static final Logger log = LoggerFactory.getLogger(OpenManageBookCtrl.class);

	@Wire
	Listbox bookList;
	@Wire
	Button open;
	@Wire
	Button delete;
	@Wire
	Button upload;
	
	BookRepository repo = BookRepositoryFactory.getInstance().getRepository();
	BookManager bookManager = BookManagerImpl.getInstance(repo);
	CollaborationInfo collaborationInfo = CollaborationInfoImpl.getInstance();
	ListModelList<Map<String,Object>> bookListModel = new ListModelList<Map<String,Object>>();
	
	public static void show(EventListener<DlgCallbackEvent> callback) {
		Map arg = newArg(callback);
		
		Window comp = (Window)Executions.createComponents(URI, null, arg);
		comp.doModal();
		return;
	}
	
	@Override
	public void doAfterCompose(Window comp) throws Exception {
		super.doAfterCompose(comp);
		reloadBookModel();
	}
	
	private void reloadBookModel(){
		bookListModel = new ListModelList<Map<String,Object>>();
		DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd");
		for(BookInfo info : repo.list()){
			Map<String,Object> data = new HashMap<String,Object>();
			data.put("name", info.getName());
			data.put("lastmodify", dateFormat.format(info.getLastModify()));
			data.put("bookinfo", info);
			bookListModel.add(data);
		}
		bookList.setModel(bookListModel);
		updateButtonState();
	}
	
	@Listen("onClick=#open; onDoubleClick=#bookList")
	public void onOpen(){
		Map<String,Object> selection = (Map<String,Object>)UiUtil.getSingleSelection(bookListModel);
		if(selection==null){
			UiUtil.showWarnMessage("Please select a book first");
			return;
		}
		BookInfo bookInfo = (BookInfo) selection.get("bookinfo");
		postCallback(ON_OPEN, newMap(newEntry(ARG_BOOKINFO, bookInfo), newEntry(ARG_BOOK, loadBook(bookInfo))));
		detach();
	}
	
	@Listen("onSelect=#bookList")
	public void onSelect(){
		updateButtonState();
	}
	
	private void updateButtonState(){
		boolean selected = bookListModel.getSelection().size()>0;

		open.setDisabled(!selected);
		delete.setDisabled(!selected || Configuration.isRepositoryReadonly());
		upload.setDisabled(Configuration.isRepositoryReadonly());
	}
	
	@Listen("onClick=#delete")
	public void onDelete(){
        if (Configuration.isRepositoryReadonly()) {
            return;
        }
        Map<String, Object> selection = (Map<String, Object>) UiUtil.getSingleSelection(bookListModel);
        if (selection == null) {
            UiUtil.showWarnMessage("Please select a book first");
            return;
        }


        synchronized (bookManager) {
            final BookInfo bookinfo = (BookInfo) selection.get("bookinfo");
            String bookName = bookinfo.getName();
            String message = "";
            if (bookManager.isBookAttached(bookinfo)) {
                String users = Arrays.toString(collaborationInfo.getUsedUsernames(bookName).toArray());
                message += "Book \"" + bookinfo.getName() + "\" is in used by " + users + ".\n";
            }

            message += "Do you want to delete file \"" + bookName + "\" ?";
            Messagebox.show(message, "Keikai Spreadsheet",
                    Messagebox.OK + Messagebox.CANCEL, Messagebox.INFORMATION, new SerializableEventListener<Event>() {
                        private static final long serialVersionUID = 4698610847862970542L;

                        @Override
                        public void onEvent(Event event) throws Exception {
                            if (event.getData().equals(Messagebox.OK)) {
                                bookManager.deleteBook(bookinfo);
                                reloadBookModel();
                            }
                        }
                    });
        }
	}
	
	@Listen("onClick=#cancel;onCancel=#openBookDlg")
	public void onCancel(){
		detach();
	}
	
	@Listen("onClick=#upload")
	public void onUpload(){
		if(Configuration.isRepositoryReadonly()){
			return;
		}
		Fileupload.get(5,new SerializableEventListener<UploadEvent>() {
			private static final long serialVersionUID = -7772499154678293597L;

			public void onEvent(UploadEvent event) throws Exception {
				Media[] medias = event.getMedias();
				String finalName = null;
				if (medias == null || medias.length <=0)
					return;
				for (Media m : event.getMedias()) {
					if (m.isBinary()) {
						InputStream is = null;
						is = m.getStreamData();
						String name = m.getName();
						Book book = Importers.getImporter().imports(is, name);
						name = BookUtil.appendExtension(name, book);
						name = BookUtil.suggestFileName(name, book, repo);
						BookInfo bookInfo = repo.saveAs(name, book);
						finalName = bookInfo.getName();
					}
				}

				reloadBookModel();
				Iterator<Map<String, Object>> iter = bookListModel.iterator();
				while (iter.hasNext()) {
					Map<String, Object> data = iter.next();
					if (data.get("name").equals(finalName)) {
						bookListModel.addToSelection(data);
						updateButtonState();
						break;
					}
				}
			}
		});
	}
	
	private Book loadBook(BookInfo bookInfo) {
		Book book = null;
		try {
			book = bookManager.readBook(bookInfo);
		} catch (IOException e) {
			log.error(e.getMessage(),e);
			UiUtil.showWarnMessage("Can't load the specified book:" + bookInfo.getName());
		}
		return book;
	}
	
	static private class MapAttrComparator implements Comparator<Map<String, Object>>, Serializable {
		private static final long serialVersionUID = -2889285276678010655L;
		private final boolean _asc;
		private final String _attr;
		
		public MapAttrComparator(boolean asc,String attr) {
			_asc = asc;
			_attr = attr;
		}

		@Override
		public int compare(Map<String, Object> o1, Map<String, Object> o2) {
			if(_asc){
				return o1.get(_attr).toString().compareTo(o2.get(_attr).toString());
			}else{
				return o2.get(_attr).toString().compareTo(o1.get(_attr).toString());
			}
		}
	}
	
	public Comparator<Map<String, Object>> getBookNameDescComparator() {
		return new MapAttrComparator(false, "name");
	}

	public Comparator<Map<String, Object>> getBookNameAscComparator() {
		return new MapAttrComparator(true, "name");
	}

	public Comparator<Map<String, Object>> getBookDateDescComparator() {
		return new MapAttrComparator(false, "lastmodify");
	}

	public Comparator<Map<String, Object>> getBookDateAscComparator() {
		return new MapAttrComparator(true, "lastmodify");
	}
	
}
